import requests
import json
import base64
import pickle
import time
import sys
import numpy as np

class Client():
    def __init__(self):
        pass
    def set_data(self,name:str,data:object):
        t0=time.time()
        url = "http://localhost:4000/jsonrpc"
        data_b64=base64.b64encode(pickle.dumps(data)).decode('ascii')
        # Example echo method
        
        payload = {
            "method": "set_data",
            "params": [name,data_b64],
            "jsonrpc": "2.0",
            "id": 0,
        }
        response = requests.post(url, json=payload).json()
        t1=time.time()
        print(response)
        print('set value,time elapsed',t1-t0,'size is %f MB'%(sys.getsizeof(data_b64)*1.0/(1024*1024*1.0)),'\n\n')

    def get_data(self,name:str):
        t0=time.time()
        url = "http://localhost:4000/jsonrpc"

        # Example echo method
        payload = {
            "method": "get_data",
            "params": [name],
            "jsonrpc": "2.0",
            "id": 0,
        }
        response = requests.post(url, json=payload).json()
        t1=time.time()
        resp_bytes = response['result'].encode('ascii')
        pkl_bytes = base64.b64decode(resp_bytes)
        data = pickle.loads(pkl_bytes)
        t2 = time.time()
        hash(response['result'])
        t3 = time.time()
        print('get value,time elapsed:\n','request:',t1-t0,'total:',t2-t0,'\ndecoding:',t2-t1,'hashing:',t3-t2)
        print(data,'\n','size is %f MB'%(sys.getsizeof(response['result'])*1.0/(1024*1024*1.0)))
        print('')
        return data



if __name__ == "__main__":
    c=Client()
    c.set_data('a','hhhhhhhhhhhhhhhhhh')
    c.get_data('a')
    c.set_data('b',np.empty(shape=(1024*2,1024*2,3),dtype=np.uint16))# 模拟，传输一张较大的图片。
    print(c.get_data('b'))


""" Example of json-rpc usage with Wergzeug and requests.
NOTE: there are no Werkzeug and requests in dependencies of json-rpc.
NOTE: server handles all url paths the same way (there are no different urls).
"""

# 依赖：werkzeug,requests
# 对于100MB以下数据的存储速度较快。32M大小的文件，访问时间大约0.43s。
# 支持pypy，但是速度提升不明显。
# 可以传输：np.ndarray,pd.DataFrame等数组。
# 原理：
#get_data:将base64数据嵌入到json中发送给客户端，客户端用base64解码之后，再通过pickle还原对象。
#set_data: 在客户端将数据pickle为二进制对象之后，进行base64化，然后嵌入到json中发送到服务端。
#                服务端直接存储base64的数据。
#编码解码过程都发生在客户端。
# 使用包：pip install json-rpc,python3以上可用。

from werkzeug.wrappers import Request, Response
from werkzeug.serving import run_simple

from jsonrpc import JSONRPCResponseManager, dispatcher
import time


@dispatcher.add_method
def foobar(**kwargs):
    return kwargs["foo"] + kwargs["bar"]

class Server():
    def __init__(self):
        self.data_dic={}# 存储的是data的base64值。
           
    def set_data(self,name:str,data_b64):
        '''如果没有数据，就添加数据。
        如果有数据，就替换数据。
        '''
        self.data_dic[name]=data_b64
        print(self.data_dic.keys())
        return "set data ok!"

    def get_data(self,name:str):
        return self.data_dic[name]
    
    @Request.application
    def application(self,request):
        # Dispatcher is dictionary {<method_name>: callable}
        t0=time.time()
        global data
        dispatcher["add"] = lambda a, b: encoded_data
        dispatcher['set_data'] = lambda name,data: self.set_data(data_b64=data,name=name)
        dispatcher['get_data'] = lambda name:self.get_data(name=name)
        
        response = JSONRPCResponseManager.handle(
            request.get_data(cache=False, as_text=True), dispatcher)
        r = Response(response.json, mimetype='application/json')
        t1=time.time()
        print(t1-t0)
        return r
    
    



if __name__ == '__main__':
    s=Server()
    run_simple('localhost', 4000, s.application)
    


from setuptools import setup,find_packages
setup(name='pyjrdatahost',
      version='0.1',
      description='python data server.',
      url='https://gitee.com/hzy15610046011/python_data_host',
      author='Zhanyi Hou',
      author_email='3120388018@qq.com',
      license='MuLan2.0',
      packages=find_packages(),
      include_package_data = True
      )
